@@ -21,8 +21,11 @@ import threading
 import config
 import cups
 import cupspk
+import gi
 from gi.repository import GLib
+gi.require_version('Gdk', '3.0')
 from gi.repository import Gdk
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 import os
 from errordialogs import *
@@ -41,10 +44,12 @@ class AuthDialog(Gtk.Dialog):
                   flags=Gtk.DialogFlags.MODAL,
                   buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                            Gtk.STOCK_OK, Gtk.ResponseType.OK),
-                  auth_info_required=['username', 'password'],
+                  auth_info_required=None,
                   allow_remember=False):
-        if title == None:
+        if title is None:
             title = _("Authentication")
+        if auth_info_required is None:
+            auth_info_required = ['username', 'password']
         Gtk.Dialog.__init__ (self, title, parent, flags, buttons)
         self.auth_info_required = auth_info_required
         self.set_default_response (Gtk.ResponseType.OK)
@@ -62,23 +67,27 @@ class AuthDialog(Gtk.Dialog):
         vbox.pack_start (self.prompt_label, False, False, 0)
 
         num_fields = len (auth_info_required)
-        table = Gtk.Table (n_rows=num_fields, n_columns=2)
-        table.set_row_spacings (6)
-        table.set_col_spacings (6)
+        grid = Gtk.Grid()
+        grid.insert_row(num_fields)
+        grid.insert_column(2)
+        grid.set_row_spacing (6)
+        grid.set_column_spacing (6)
 
         self.field_entry = []
         for i in range (num_fields):
             field = auth_info_required[i]
             label = Gtk.Label (label=_(self.AUTH_FIELD.get (field, field)))
             label.set_alignment (0, 0.5)
-            table.attach (label, 0, 1, i, i + 1)
+            grid.attach (label, 0, 1, i, i + 1)
+            grid.attach (label, 0, i, 1, 1)
             entry = Gtk.Entry ()
             entry.set_visibility (field != 'password')
-            table.attach (entry, 1, 2, i, i + 1, 0, 0)
+            grid.attach (entry, 1, 2, i, i + 1, 0, 0)
+            grid.attach (entry, 1, i, 1, 1)
             self.field_entry.append (entry)
 
         self.field_entry[num_fields - 1].set_activates_default (True)
-        vbox.pack_start (table, False, False, 0)
+        vbox.pack_start (grid, False, False, 0)
         hbox.pack_start (vbox, False, False, 0)
         self.vbox.pack_start (hbox, False, False, 0)
 
@@ -122,13 +131,13 @@ class _AuthInfoCache:
         self.creds = dict() # by (host,port)
 
     def cache_auth_info (self, data, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         self.creds[(host,port)] = data
 
     def lookup_auth_info (self, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         try:
@@ -137,7 +146,7 @@ class _AuthInfoCache:
             return None
 
     def remove_auth_info (self, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         try:
@@ -150,11 +159,11 @@ global_authinfocache = _AuthInfoCache ()
 class Connection:
     def __init__ (self, parent=None, try_as_root=True, lock=False,
                   host=None, port=None, encryption=None):
-        if host != None:
+        if host is not None:
             cups.setServer (host)
-        if port != None:
+        if port is not None:
             cups.setPort (port)
-        if encryption != None:
+        if encryption is not None:
             cups.setEncryption (encryption)
 
         self._use_password = ''
@@ -299,7 +308,7 @@ class Connection:
         except IndexError:
             msg = _("CUPS server error")
 
-        d = Gtk.MessageDialog (parent=self._parent,
+        d = Gtk.MessageDialog (transient_for=self._parent,
                                modal=True, destroy_with_parent=True,
                                message_type=Gtk.MessageType.ERROR,
                                buttons=Gtk.ButtonsType.NONE,
@@ -339,7 +348,7 @@ class Connection:
         self._passes += 1
 
         creds = global_authinfocache.lookup_auth_info (host=self._server, port=self._port)
-        if creds != None:
+        if creds is not None:
             if (creds[0] != 'root' or self._try_as_root):
                 (self._use_user, self._use_password) = creds
             del creds
@@ -440,7 +449,7 @@ class Connection:
     def _show_not_authorized_dialog (self):
         if self._lock:
             Gdk.threads_enter ()
-        d = Gtk.MessageDialog (parent=self._parent,
+        d = Gtk.MessageDialog (transient_for=self._parent,
                                modal=True, destroy_with_parent=True,
                                message_type=Gtk.MessageType.ERROR,
                                buttons=Gtk.ButtonsType.CLOSE)
